@@ -40,7 +40,6 @@
 #define SPIRVTOLLVMDBGTRAN_H
 
 #include "SPIRVInstruction.h"
-#include "SPIRVModule.h"
 
 #include "llvm/IR/DIBuilder.h"
 #include "llvm/IR/DebugLoc.h"
@@ -173,6 +172,7 @@ private:
     return nullptr;
   }
   const std::string &getString(const SPIRVId Id);
+  std::string findModuleProducer();
 };
 } // namespace SPIRV
 
