@@ -43,7 +43,7 @@
 #include "llvm/IR/InstVisitor.h"
 #include "llvm/Pass.h"
 
-#include <cstring>
+#include <string>
 
 namespace SPIRV {
 class SPIRVToOCL : public ModulePass, public InstVisitor<SPIRVToOCL> {
@@ -98,10 +98,41 @@ public:
   ///  intel_sub_group_media_block_write
   void visitCallSPIRVImageMediaBlockBuiltin(CallInst *CI, Op OC);
 
+  /// Transform __spirv_*Convert_R{ReturnType}{_sat}{_rtp|_rtn|_rtz|_rte} to
+  /// convert_{ReturnType}_{sat}{_rtp|_rtn|_rtz|_rte}
+  /// example:  <2 x i8> __spirv_SatConvertUToS(<2 x i32>) =>
+  ///   convert_uchar2_sat(int2)
+  void visitCallSPIRVCvtBuiltin(CallInst *CI, Op OC, StringRef DemangledName);
+
+  /// Transform
+  ///   __spirv_AsyncGroupCopy(ScopeWorkGroup, dst, src, n, stride, event)
+  ///   => async_work_group_strided_copy(dst, src, n, stride, event)
+  void visitCallAsyncWorkGroupCopy(CallInst *CI, Op OC);
+
+  /// Transform __spirv_GroupWaitEvents(Scope, NumEvents, EventsList)
+  ///   => wait_group_events(NumEvents, EventsList)
+  void visitCallGroupWaitEvents(CallInst *CI, Op OC);
+
+  /// Transform __spirv_ImageSampleExplicitLod__{ReturnType} to read_imade
+  void visitCallSPIRVImageSampleExplicitLodBuiltIn(CallInst *CI, Op OC);
+
   /// Transform __spirv_* builtins to OCL 2.0 builtins.
   /// No change with arguments.
   void visitCallSPIRVBuiltin(CallInst *CI, Op OC);
 
+  /// Transform __spirv_ocl* instructions (OpenCL Extended Instruction Set)
+  /// to OpenCL builtins.
+  void visitCallSPIRVOCLExt(CallInst *CI, OCLExtOpKind Kind);
+
+  /// Transform __spirv_ocl_vstore* to corresponding vstore OpenCL instruction
+  void visitCallSPIRVVStore(CallInst *CI, OCLExtOpKind Kind);
+
+  /// Transform __spirv_ocl_vloadn to OpenCL vload[2|4|8|16]
+  void visitCallSPIRVVLoadn(CallInst *CI, OCLExtOpKind Kind);
+
+  /// Transform __spirv_ocl_printf to (i8 addrspace(2)*, ...) @printf
+  void visitCallSPIRVPrintf(CallInst *CI, OCLExtOpKind Kind);
+
   /// Get prefix work_/sub_ for OCL group builtin functions.
   /// Assuming the first argument of \param CI is a constant integer for
   /// workgroup/subgroup scope enums.
@@ -150,7 +181,7 @@ private:
   std::string getNonUniformArithmeticBuiltinName(CallInst *CI, Op OC);
   /// Transform ballot bit count opcode to corresponding OpenCL function name,
   /// example: GroupNonUniformBallotBitCount(Reduce) =>
-  /// group_ballot_bit_count_iadd => sub_​group_​ballot_​bit_​count
+  /// group_ballot_bit_count_iadd => sub_group_ballot_bit_count
   std::string getBallotBuiltinName(CallInst *CI, Op OC);
   /// Transform group opcode to corresponding OpenCL function name
   std::string groupOCToOCLBuiltinName(CallInst *CI, Op OC);
