@@ -46,10 +46,12 @@
 #include <iostream>
 #include <set>
 #include <string>
-#include <unordered_map>
-#include <unordered_set>
 #include <vector>
 
+namespace llvm {
+class APInt;
+} // namespace llvm
+
 namespace SPIRV {
 
 template <Op> class SPIRVConstantBase;
@@ -164,7 +166,6 @@ public:
   virtual void setMemoryModel(SPIRVMemoryModelKind) = 0;
   virtual void setName(SPIRVEntry *, const std::string &) = 0;
   virtual void setSourceLanguage(SourceLanguage, SPIRVWord) = 0;
-  virtual void optimizeDecorates() = 0;
   virtual void setAutoAddCapability(bool E) { AutoAddCapability = E; }
   virtual void setValidateCapability(bool E) { ValidateCapability = E; }
   virtual void setAutoAddExtensions(bool E) { AutoAddExtensions = E; }
@@ -235,6 +236,7 @@ public:
   virtual SPIRVTypePointer *addPointerType(SPIRVStorageClassKind,
                                            SPIRVType *) = 0;
   virtual SPIRVTypeStruct *openStructType(unsigned, const std::string &) = 0;
+  virtual SPIRVEntry *addTypeStructContinuedINTEL(unsigned NumMembers) = 0;
   virtual void closeStructType(SPIRVTypeStruct *, bool) = 0;
   virtual SPIRVTypeVector *addVectorType(SPIRVType *, SPIRVWord) = 0;
   virtual SPIRVTypeVoid *addVoidType() = 0;
@@ -251,10 +253,18 @@ public:
   // Constants creation functions
   virtual SPIRVValue *
   addCompositeConstant(SPIRVType *, const std::vector<SPIRVValue *> &) = 0;
+  virtual SPIRVEntry *
+  addCompositeConstantContinuedINTEL(const std::vector<SPIRVValue *> &) = 0;
+  virtual SPIRVValue *
+  addSpecConstantComposite(SPIRVType *Ty,
+                           const std::vector<SPIRVValue *> &Elements) = 0;
+  virtual SPIRVEntry *
+  addSpecConstantCompositeContinuedINTEL(const std::vector<SPIRVValue *> &) = 0;
   virtual SPIRVValue *addConstFunctionPointerINTEL(SPIRVType *Ty,
                                                    SPIRVFunction *F) = 0;
   virtual SPIRVValue *addConstant(SPIRVValue *) = 0;
   virtual SPIRVValue *addConstant(SPIRVType *, uint64_t) = 0;
+  virtual SPIRVValue *addConstant(SPIRVType *, llvm::APInt) = 0;
   virtual SPIRVValue *addSpecConstant(SPIRVType *, uint64_t) = 0;
   virtual SPIRVValue *addDoubleConstant(SPIRVTypeFloat *, double) = 0;
   virtual SPIRVValue *addFloatConstant(SPIRVTypeFloat *, float) = 0;
@@ -293,6 +303,7 @@ public:
                                        SPIRVInstruction * = nullptr) = 0;
   virtual SPIRVEntry *addDebugInfo(SPIRVWord, SPIRVType *,
                                    const std::vector<SPIRVWord> &) = 0;
+  virtual SPIRVEntry *addModuleProcessed(const std::string &) = 0;
   virtual void addCapability(SPIRVCapabilityKind) = 0;
   template <typename T> void addCapabilities(const T &Caps) {
     for (auto I : Caps)
@@ -375,6 +386,10 @@ public:
                           std::vector<SPIRVWord> LoopControlParameters,
                           SPIRVBasicBlock *BB) = 0;
   virtual SPIRVInstruction *
+  addFixedPointIntelInst(Op OC, SPIRVType *ResTy, SPIRVValue *Input,
+                         const std::vector<SPIRVWord> &Ops,
+                         SPIRVBasicBlock *BB) = 0;
+  virtual SPIRVInstruction *
   addArbFloatPointIntelInst(Op OC, SPIRVType *ResTy, SPIRVValue *InA,
                             SPIRVValue *InB, const std::vector<SPIRVWord> &Ops,
                             SPIRVBasicBlock *BB) = 0;
@@ -460,20 +475,12 @@ public:
     return TranslationOpts.isAllowedToUseExtension(RequestedExtension);
   }
 
-  virtual bool
-  isAllowedToUseExtensions(const SPIRVExtSet &RequestedExtensions) const final {
-    for (const auto &Ext : RequestedExtensions) {
-      if (!TranslationOpts.isAllowedToUseExtension(Ext))
-        return false;
-    }
-
-    return true;
-  }
-
   virtual bool isGenArgNameMDEnabled() const final {
     return TranslationOpts.isGenArgNameMDEnabled();
   }
 
+  virtual std::vector<SPIRVModuleProcessed *> getModuleProcessedVec() = 0;
+
   bool getSpecializationConstant(SPIRVWord SpecId, uint64_t &ConstValue) {
     return TranslationOpts.getSpecializationConstant(SpecId, ConstValue);
   }
@@ -490,6 +497,10 @@ public:
     return TranslationOpts.allowExtraDIExpressions();
   }
 
+  bool shouldReplaceLLVMFmulAddWithOpenCLMad() const noexcept {
+    return TranslationOpts.shouldReplaceLLVMFmulAddWithOpenCLMad();
+  }
+
   SPIRVExtInstSetKind getDebugInfoEIS() const {
     switch (TranslationOpts.getDebugInfoEIS()) {
     case DebugInfoEIS::SPIRV_Debug:
@@ -502,6 +513,10 @@ public:
     }
   }
 
+  BIsRepresentation getDesiredBIsRepresentation() const {
+    return TranslationOpts.getDesiredBIsRepresentation();
+  }
+
   // I/O functions
   friend spv_ostream &operator<<(spv_ostream &O, SPIRVModule &M);
   friend std::istream &operator>>(std::istream &I, SPIRVModule &M);
