@@ -294,6 +294,7 @@ _SPIRV_OP(AtomicFlagClear, 319)
 _SPIRV_OP(TypePipeStorage, 322)
 _SPIRV_OP(ConstantPipeStorage, 323)
 _SPIRV_OP(CreatePipeFromPipeStorage, 324)
+_SPIRV_OP(ModuleProcessed, 330)
 _SPIRV_OP(GroupNonUniformElect, 333)
 _SPIRV_OP(GroupNonUniformAll, 334)
 _SPIRV_OP(GroupNonUniformAny, 335)
@@ -326,7 +327,6 @@ _SPIRV_OP(GroupNonUniformBitwiseXor, 361)
 _SPIRV_OP(GroupNonUniformLogicalAnd, 362)
 _SPIRV_OP(GroupNonUniformLogicalOr, 363)
 _SPIRV_OP(GroupNonUniformLogicalXor, 364)
-_SPIRV_OP(Forward, 1024)
 _SPIRV_OP(SubgroupShuffleINTEL, 5571)
 _SPIRV_OP(SubgroupShuffleDownINTEL, 5572)
 _SPIRV_OP(SubgroupShuffleUpINTEL, 5573)
@@ -342,8 +342,6 @@ _SPIRV_OP(FunctionPointerCallINTEL, 5601)
 _SPIRV_OP(AsmTargetINTEL, 5609)
 _SPIRV_OP(AsmINTEL, 5610)
 _SPIRV_OP(AsmCallINTEL, 5611)
-_SPIRV_OP(AssumeTrueINTEL, 5630)
-_SPIRV_OP(ExpectINTEL, 5631)
 _SPIRV_OP(VmeImageINTEL, 5699)
 _SPIRV_OP(TypeVmeImageINTEL, 5700)
 _SPIRV_OP(TypeAvcImePayloadINTEL, 5701)
@@ -468,6 +466,9 @@ _SPIRV_OP(SubgroupAvcSicGetIpeChromaModeINTEL, 5813)
 _SPIRV_OP(SubgroupAvcSicGetPackedSkcLumaCountThresholdINTEL, 5814)
 _SPIRV_OP(SubgroupAvcSicGetPackedSkcLumaSumThresholdINTEL, 5815)
 _SPIRV_OP(SubgroupAvcSicGetInterRawSadsINTEL, 5816)
+_SPIRV_OP(VariableLengthArrayINTEL, 5818)
+_SPIRV_OP(SaveMemoryINTEL, 5819)
+_SPIRV_OP(RestoreMemoryINTEL, 5820)
 _SPIRV_OP(ArbitraryFloatSinCosPiINTEL, 5840)
 _SPIRV_OP(ArbitraryFloatCastINTEL, 5841)
 _SPIRV_OP(ArbitraryFloatCastFromIntINTEL, 5842)
@@ -510,9 +511,24 @@ _SPIRV_OP(ArbitraryFloatPowINTEL, 5880)
 _SPIRV_OP(ArbitraryFloatPowRINTEL, 5881)
 _SPIRV_OP(ArbitraryFloatPowNINTEL, 5882)
 _SPIRV_OP(LoopControlINTEL, 5887)
+_SPIRV_OP(FixedSqrtINTEL, 5923)
+_SPIRV_OP(FixedRecipINTEL, 5924)
+_SPIRV_OP(FixedRsqrtINTEL, 5925)
+_SPIRV_OP(FixedSinINTEL, 5926)
+_SPIRV_OP(FixedCosINTEL, 5927)
+_SPIRV_OP(FixedSinCosINTEL, 5928)
+_SPIRV_OP(FixedSinPiINTEL, 5929)
+_SPIRV_OP(FixedCosPiINTEL, 5930)
+_SPIRV_OP(FixedSinCosPiINTEL, 5931)
+_SPIRV_OP(FixedLogINTEL, 5932)
+_SPIRV_OP(FixedExpINTEL, 5933)
 _SPIRV_OP(PtrCastToCrossWorkgroupINTEL, 5934)
 _SPIRV_OP(CrossWorkgroupCastToPtrINTEL, 5938)
 _SPIRV_OP(ReadPipeBlockingINTEL, 5946)
 _SPIRV_OP(WritePipeBlockingINTEL, 5947)
 _SPIRV_OP(FPGARegINTEL, 5949)
+_SPIRV_OP(AtomicFAddEXT, 6035)
 _SPIRV_OP(TypeBufferSurfaceINTEL, 6086)
+_SPIRV_OP(TypeStructContinuedINTEL, 6090)
+_SPIRV_OP(ConstantCompositeContinuedINTEL, 6091)
+_SPIRV_OP(SpecConstantCompositeContinuedINTEL, 6092)
