@@ -48,8 +48,6 @@
 #include "SPIRVStream.h"
 
 #include <cassert>
-#include <iostream>
-#include <map>
 #include <tuple>
 #include <vector>
 
@@ -167,15 +165,15 @@ public:
     }
     return CV;
   }
-  SPIRVExtSet getRequiredExtensions() const override {
+  llvm::Optional<ExtensionID> getRequiredExtension() const override {
     switch (BitWidth) {
     case 8:
     case 16:
     case 32:
     case 64:
-      return SPIRVExtSet();
+      return {};
     default:
-      return getSet(ExtensionID::SPV_INTEL_arbitrary_precision_integers);
+      return ExtensionID::SPV_INTEL_arbitrary_precision_integers;
     }
   }
 
@@ -183,7 +181,12 @@ protected:
   _SPIRV_DEF_ENCDEC3(Id, BitWidth, IsSigned)
   void validate() const override {
     SPIRVEntry::validate();
-    assert(BitWidth > 1 && BitWidth <= 64 && "Invalid bit width");
+    assert(BitWidth > 1 &&
+           (BitWidth <= 64 ||
+            (Module->isAllowedToUseExtension(
+                 ExtensionID::SPV_INTEL_arbitrary_precision_integers) &&
+             BitWidth <= 2048)) &&
+           "Invalid bit width");
   }
 
 private:
@@ -481,7 +484,7 @@ inline bool operator<(const SPIRVTypeImageDescriptor &A,
 class SPIRVTypeImage : public SPIRVType {
 public:
   const static Op OC = OpTypeImage;
-  const static SPIRVWord FixedWC = 9;
+  constexpr static SPIRVWord FixedWC = 9;
   SPIRVTypeImage(SPIRVModule *M, SPIRVId TheId, SPIRVId TheSampledType,
                  const SPIRVTypeImageDescriptor &TheDesc)
       : SPIRVType(M, FixedWC, OC, TheId), SampledType(TheSampledType),
@@ -616,11 +619,17 @@ protected:
 
 class SPIRVTypeStruct : public SPIRVType {
 public:
+  const static Op OC = OpTypeStruct;
+  // There are always 2 words in this instruction except member types:
+  // 1) WordCount + OpCode
+  // 2) Result Id
+  constexpr static SPIRVWord FixedWC = 2;
+  using ContinuedInstType = typename InstToContinued<OC>::Type;
   // Complete constructor
   SPIRVTypeStruct(SPIRVModule *M, SPIRVId TheId,
                   const std::vector<SPIRVType *> &TheMemberTypes,
                   const std::string &TheName)
-      : SPIRVType(M, 2 + TheMemberTypes.size(), OpTypeStruct, TheId) {
+      : SPIRVType(M, FixedWC + TheMemberTypes.size(), OC, TheId) {
     MemberTypeIdVec.resize(TheMemberTypes.size());
     for (auto &T : TheMemberTypes)
       MemberTypeIdVec.push_back(T->getId());
@@ -629,20 +638,27 @@ public:
   }
   SPIRVTypeStruct(SPIRVModule *M, SPIRVId TheId, unsigned NumMembers,
                   const std::string &TheName)
-      : SPIRVType(M, 2 + NumMembers, OpTypeStruct, TheId) {
+      : SPIRVType(M, FixedWC + NumMembers, OC, TheId) {
     Name = TheName;
     validate();
     MemberTypeIdVec.resize(NumMembers);
   }
   // Incomplete constructor
-  SPIRVTypeStruct() : SPIRVType(OpTypeStruct) {}
+  SPIRVTypeStruct() : SPIRVType(OC) {}
 
   SPIRVWord getMemberCount() const { return MemberTypeIdVec.size(); }
   SPIRVType *getMemberType(size_t I) const {
     return static_cast<SPIRVType *>(getEntry(MemberTypeIdVec[I]));
   }
   void setMemberType(size_t I, SPIRVType *Ty) {
-    MemberTypeIdVec[I] = Ty->getId();
+    if (I >= MemberTypeIdVec.size() && !ContinuedInstructions.empty()) {
+      const size_t MaxNumElements = MaxWordCount - FixedWC;
+      I -= MaxNumElements; // Remove operands that included into OpTypeStruct
+      ContinuedInstructions[I / MaxNumElements]->setElementId(
+          I % MaxNumElements, Ty->getId());
+    } else {
+      MemberTypeIdVec[I] = Ty->getId();
+    }
   }
 
   bool isPacked() const;
@@ -650,23 +666,51 @@ public:
 
   void setWordCount(SPIRVWord WordCount) override {
     SPIRVType::setWordCount(WordCount);
-    MemberTypeIdVec.resize(WordCount - 2);
+    MemberTypeIdVec.resize(WordCount - FixedWC);
   }
 
+  // TODO: Should we attach operands of continued instructions as well?
   std::vector<SPIRVEntry *> getNonLiteralOperands() const override {
     std::vector<SPIRVEntry *> Operands(MemberTypeIdVec.size());
     for (size_t I = 0, E = MemberTypeIdVec.size(); I < E; ++I)
       Operands[I] = getEntry(MemberTypeIdVec[I]);
     return Operands;
   }
+  void addContinuedInstruction(ContinuedInstType Inst) {
+    ContinuedInstructions.push_back(Inst);
+  }
+
+  void encodeChildren(spv_ostream &O) const override {
+    O << SPIRVNL();
+    for (auto &I : ContinuedInstructions)
+      O << *I;
+  }
+
+  std::vector<ContinuedInstType> getContinuedInstructions() {
+    return ContinuedInstructions;
+  }
 
 protected:
-  _SPIRV_DEF_ENCDEC2(Id, MemberTypeIdVec)
+  void encode(spv_ostream &O) const override {
+    getEncoder(O) << Id << MemberTypeIdVec;
+  }
+
+  void decode(std::istream &I) override {
+    SPIRVDecoder Decoder = getDecoder(I);
+    Decoder >> Id >> MemberTypeIdVec;
+    Module->add(this);
+
+    for (SPIRVEntry *E : Decoder.getContinuedInstructions(ContinuedOpCode)) {
+      addContinuedInstruction(static_cast<ContinuedInstType>(E));
+    }
+  }
 
   void validate() const override { SPIRVEntry::validate(); }
 
 private:
   std::vector<SPIRVId> MemberTypeIdVec; // Member Type Ids
+  std::vector<ContinuedInstType> ContinuedInstructions;
+  const spv::Op ContinuedOpCode = InstToContinued<OC>::OpCode;
 };
 
 class SPIRVTypeFunction : public SPIRVType {
@@ -843,8 +887,8 @@ public:
     return getVec(CapabilityVectorComputeINTEL);
   }
 
-  SPIRVExtSet getRequiredExtensions() const override {
-    return getSet(ExtensionID::SPV_INTEL_vector_compute);
+  llvm::Optional<ExtensionID> getRequiredExtension() const override {
+    return {ExtensionID::SPV_INTEL_vector_compute};
   }
 
   bool hasAccessQualifier() const { return AccessKind.hasValue(); }
@@ -893,8 +937,8 @@ public:
     return getVec(CapabilitySubgroupAvcMotionEstimationINTEL);
   }
 
-  SPIRVExtSet getRequiredExtensions() const override {
-    return getSet(ExtensionID::SPV_INTEL_device_side_avc_motion_estimation);
+  llvm::Optional<ExtensionID> getRequiredExtension() const override {
+    return ExtensionID::SPV_INTEL_device_side_avc_motion_estimation;
   }
 
 protected:
@@ -948,8 +992,8 @@ public:
     return getVec(CapabilitySubgroupAvcMotionEstimationINTEL);
   }
 
-  SPIRVExtSet getRequiredExtensions() const override {
-    return getSet(ExtensionID::SPV_INTEL_device_side_avc_motion_estimation);
+  llvm::Optional<ExtensionID> getRequiredExtension() const override {
+    return ExtensionID::SPV_INTEL_device_side_avc_motion_estimation;
   }
 
   SPIRVValue *getOperand() { return getValue(Opn); }
