@@ -47,9 +47,11 @@
 #include "SPIRVEntry.h"
 #include "SPIRVType.h"
 
+namespace llvm {
+class APInt;
+} // namespace llvm
+
 #include <iostream>
-#include <map>
-#include <memory>
 
 namespace SPIRV {
 
@@ -121,12 +123,13 @@ public:
     return Type->getRequiredCapability();
   }
 
-  SPIRVExtSet getRequiredExtensions() const override {
-    SPIRVExtSet EV;
+  llvm::Optional<ExtensionID> getRequiredExtension() const override {
+    llvm::Optional<ExtensionID> EV;
     if (!hasType())
       return EV;
-    EV = Type->getRequiredExtensions();
-    assert(!Module || Module->isAllowedToUseExtensions(EV));
+    EV = Type->getRequiredExtension();
+    assert(Module &&
+           (!EV.hasValue() || Module->isAllowedToUseExtension(EV.getValue())));
     return EV;
   }
 
@@ -147,6 +150,9 @@ public:
     recalculateWordCount();
     validate();
   }
+  // Incomplete constructor for AP integer constant
+  SPIRVConstantBase(SPIRVModule *M, SPIRVType *TheType, SPIRVId TheId,
+                    llvm::APInt &TheValue);
   // Complete constructor for float constant
   SPIRVConstantBase(SPIRVModule *M, SPIRVType *TheType, SPIRVId TheId,
                     float TheValue)
@@ -168,17 +174,17 @@ public:
   uint64_t getZExtIntValue() const { return Union.UInt64Val; }
   float getFloatValue() const { return Union.FloatVal; }
   double getDoubleValue() const { return Union.DoubleVal; }
+  unsigned getNumWords() const { return NumWords; }
+  SPIRVWord *getSPIRVWords() { return Union.Words; }
 
 protected:
   void recalculateWordCount() {
-    NumWords = Type->getBitWidth() / 32;
-    if (NumWords < 1)
-      NumWords = 1;
+    NumWords = (Type->getBitWidth() + 31) / 32;
     WordCount = 3 + NumWords;
   }
   void validate() const override {
     SPIRVValue::validate();
-    assert(NumWords >= 1 && NumWords <= 2 && "Invalid constant size");
+    assert(NumWords >= 1 && NumWords <= 64 && "Invalid constant size");
   }
   void encode(spv_ostream &O) const override {
     getEncoder(O) << Type << Id;
@@ -200,7 +206,7 @@ protected:
     uint64_t UInt64Val;
     float FloatVal;
     double DoubleVal;
-    SPIRVWord Words[2];
+    SPIRVWord Words[64];
     UnionType() { UInt64Val = 0; }
   } Union;
 };
@@ -281,35 +287,71 @@ protected:
 
 template <spv::Op OC> class SPIRVConstantCompositeBase : public SPIRVValue {
 public:
+  // There are always 3 words in this instruction except constituents:
+  // 1) WordCount + OpCode
+  // 2) Result type
+  // 3) Result Id
+  constexpr static SPIRVWord FixedWC = 3;
+  using ContinuedInstType = typename InstToContinued<OC>::Type;
   // Complete constructor for composite constant
   SPIRVConstantCompositeBase(SPIRVModule *M, SPIRVType *TheType, SPIRVId TheId,
                              const std::vector<SPIRVValue *> TheElements)
-      : SPIRVValue(M, TheElements.size() + 3, OpConstantComposite, TheType,
-                   TheId) {
+      : SPIRVValue(M, TheElements.size() + FixedWC, OC, TheType, TheId) {
     Elements = getIds(TheElements);
     validate();
   }
   // Incomplete constructor
-  SPIRVConstantCompositeBase() : SPIRVValue(OpConstantComposite) {}
+  SPIRVConstantCompositeBase() : SPIRVValue(OC) {}
   std::vector<SPIRVValue *> getElements() const { return getValues(Elements); }
+
+  // TODO: Should we attach operands of continued instructions as well?
   std::vector<SPIRVEntry *> getNonLiteralOperands() const override {
     std::vector<SPIRVValue *> Elements = getElements();
     return std::vector<SPIRVEntry *>(Elements.begin(), Elements.end());
   }
 
+  std::vector<ContinuedInstType> getContinuedInstructions() {
+    return ContinuedInstructions;
+  }
+
+  void addContinuedInstruction(ContinuedInstType Inst) {
+    ContinuedInstructions.push_back(Inst);
+  }
+
+  void encodeChildren(spv_ostream &O) const override {
+    O << SPIRVNL();
+    for (auto &I : ContinuedInstructions)
+      O << *I;
+  }
+
 protected:
   void validate() const override {
     SPIRVValue::validate();
     for (auto &I : Elements)
       getValue(I)->validate();
   }
+
   void setWordCount(SPIRVWord WordCount) override {
     SPIRVEntry::setWordCount(WordCount);
-    Elements.resize(WordCount - 3);
+    Elements.resize(WordCount - FixedWC);
+  }
+
+  void encode(spv_ostream &O) const override {
+    getEncoder(O) << Type << Id << Elements;
+  }
+
+  void decode(std::istream &I) override {
+    SPIRVDecoder Decoder = getDecoder(I);
+    Decoder >> Type >> Id >> Elements;
+
+    for (SPIRVEntry *E : Decoder.getContinuedInstructions(ContinuedOpCode)) {
+      addContinuedInstruction(static_cast<ContinuedInstType>(E));
+    }
   }
-  _SPIRV_DEF_ENCDEC3(Type, Id, Elements)
 
   std::vector<SPIRVId> Elements;
+  std::vector<ContinuedInstType> ContinuedInstructions;
+  const spv::Op ContinuedOpCode = InstToContinued<OC>::OpCode;
 };
 
 using SPIRVConstantComposite = SPIRVConstantCompositeBase<OpConstantComposite>;
@@ -395,7 +437,7 @@ protected:
 
 class SPIRVForward : public SPIRVValue, public SPIRVComponentExecutionModes {
 public:
-  const static Op OC = OpForward;
+  const static Op OC = internal::OpForward;
   // Complete constructor
   SPIRVForward(SPIRVModule *TheModule, SPIRVType *TheTy, SPIRVId TheId)
       : SPIRVValue(TheModule, 0, OC, TheId) {
